#include "elementRfgun.h"


#include "mathematicalTools.h"
#include "mixedTools.h"


elementRfgun::elementRfgun() : abstractElement() 
    {
      setDefaultValues();
      setDefaults();
      elementName_ = nomdElements("rfgun");
      nbParam_ = elementName_.getElementNbParameters();
      parametersString_ = new string[nbParam_+1];
      //      parmelaOk_ = true;
    }

  void elementRfgun::setDefaultValues()
  {
    defaultLabel_ = "rfgun999";
    defaultPhaseStep_ = 1.0;
    defaultNmacrop_ = 0;
    defaultSigma_t_ = 0.0;
    defaultSigma_r_ = 0.0;
    defaultE_cin_ = 0.0;
    defaultSigma_E_ = 0.0;
    defaultNb_true_particles_ = 1.0;
  }

  void elementRfgun::setDefaults()
  {
    label_ = defaultLabel_;
    phaseStep_ = defaultPhaseStep_;
    nmacrop_ = defaultNmacrop_;
    sigma_t_ = defaultSigma_t_;
    sigma_r_ = defaultSigma_r_ ;
    E_cin_ = defaultE_cin_;
    sigma_E_ = defaultSigma_E_;
    nb_true_particles_ = defaultNb_true_particles_ ;
  }

trivaluedBool elementRfgun::is_accepted_by_software(nomDeLogiciel soft) const {
  cout << " elementRfgun : c'est le bon is_accepted_by_software " << endl;
  if ( soft == nomDeLogiciel::parmela ) return ok;
  else return error;
}

 string* elementRfgun::getParametersString() const 
  {
   int compteur = -1;
    parametersString_[++compteur] = mixedTools::intToString(nbParam_);
  parametersString_[++compteur] = label_;
    parametersString_[++compteur] = mixedTools::intToString(nmacrop_);
    parametersString_[++compteur] = mixedTools::doubleToString(sigma_t_);
    parametersString_[++compteur] = mixedTools::doubleToString(sigma_r_);
    parametersString_[++compteur] = mixedTools::doubleToString(E_cin_);
    parametersString_[++compteur] = mixedTools::doubleToString(sigma_E_);
    parametersString_[++compteur] = mixedTools::doubleToString(nb_true_particles_);
    return parametersString_;
  }

  void elementRfgun::setParametersString(string* param)
  {
    if ( param == NULL )
      {
	cerr << "  elementRfgun::setParametersString parameters empty parameter set";
	return;
      }
   int compteur = -1;
    int nbparam = atoi(param[++compteur].c_str());
    if ( nbparam != nbParam_ ) 
      {
	cerr << "  elementRfgun::setParametersString parameters do not match for a DRIFT";
	return;
      }

    label_ = param[++compteur];
    nmacrop_ = atoi(param[++compteur].c_str());
    sigma_t_ = atof(param[++compteur].c_str());
    sigma_r_ = atof(param[++compteur].c_str());
    E_cin_ = atof(param[++compteur].c_str());
    sigma_E_ = atof(param[++compteur].c_str());
    nb_true_particles_ = atof(param[++compteur].c_str());

  }

string elementRfgun::parmelaOutputFlow() const
  {
    ostringstream sortie;
    // on prend les troncatures tmax et rmax à 3 sigmas 
    sortie << "INPUT 10 /np=" << nmacrop_ << "  /sigt=" << sigma_t_ << endl;
    sortie << "  /tmax=" << 3.3*sigma_t_ << " /sigr=" << sigma_r_ << endl;
    sortie << " /rmax=" << 3.0*sigma_r_ << " /W0=" << E_cin_ << " /dw0=" << sigma_E_ << endl;
    sortie << " /dwt=" << phaseStep_ << " /ran=2" << endl;

    sortie << "SCHEFF /beami=" << -nb_true_particles_ << " /nprog=2 /point=-1.7";
 
    return sortie.str();
  }

string elementRfgun::FileOutputFlow() const
  {
    ostringstream sortie;
    // on prend les troncatures tmax et rmax à 3 sigmas 
    //    sortie << elementName_.getElementType() << endl;
    sortie << elementName_.getElementLabel() << endl;
    sortie  << label_ << endl;
    sortie <<  nmacrop_ << "  " << sigma_t_ << "  " << sigma_r_ << endl;
    sortie << E_cin_ << " " << sigma_E_ << endl;
    sortie << phaseStep_ << "  "  << nb_true_particles_ << endl; 
    return sortie.str();
  }

void elementRfgun::FileInput(ifstream& ifs)
{
  ifs >> label_;
  ifs >> nmacrop_ >> sigma_t_ >> sigma_r_;
  ifs >> E_cin_ >> sigma_E_;
  ifs >> phaseStep_ >> nb_true_particles_;
}


string elementRfgun::print() {
    string txt = "";
    
    txt += label_;
    txt += "\nnr of macroparticles : ";
    txt += mixedTools::doubleToString(nb_true_particles_);
    txt += "\nsigma_t (ps) : ";
    txt += mixedTools::doubleToString(sigma_t_);
    txt += "\nsigma_r (cm) : ";
    txt += mixedTools::doubleToString(sigma_r_);
    txt += "\ninitial kinetic energy (MeV) : ";
    txt += mixedTools::doubleToString(E_cin_);
    txt += "\nsigma_Ecin (MeV) : ";
    txt += mixedTools::doubleToString(sigma_E_);
    txt += "\ntrue number of part. in beam : ";
    txt += mixedTools::boolToString(nmacrop_);
    
    return txt;
}
